/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TaxonWidgetFilter.java,v 8f6c7d15bf61 2010/08/07 16:16:37 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android.taxonomy;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.List;
import android.widget.Filter;
import it.tidalwave.bluebill.mobile.taxonomy.TaxonFilter;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.bluebill.taxonomy.Taxon;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class TaxonWidgetFilter extends Filter
  {
    private static final String CLASS = TaxonWidgetFilter.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Nonnull
    private final List<Taxon> taxa;

    @Nonnull
    private final TaxonAdapter taxonomyAdapter;

    @Nonnull
    private int[] indexTable;

    @Nonnegative
    private int filteredCount;

    private TaxonFilter taxonFilter = new TaxonFilter();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public TaxonWidgetFilter (final @Nonnull List<Taxon> taxa,
                              final @Nonnull TaxonAdapter taxonomyAdapter)
      {
        this.taxa = taxa;
        this.taxonomyAdapter = taxonomyAdapter;
        this.filteredCount = taxa.size();
        this.indexTable = new int[taxa.size()];

        for (int i = 0; i < indexTable.length; i++)
          {
            this.indexTable[i] = i;
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnegative
    public int getCount()
      {
        return filteredCount;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Taxon getItem (final @Nonnegative int index)
      {
        return taxa.get(indexTable[index]);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    protected FilterResults performFiltering (final @Nonnull CharSequence match)
      {
        // FIXME: can be improved:
        // first, create a stack of pairs (match, FilterResults); in this way, when you delete the last character, you
        // have the previous result immediately
        // second, when you add a new character, you can filter only on the previous results
        final FilterResults results = new FilterResults();
        final int[] newIndexes = new int[taxa.size()];
        results.values = newIndexes;
        results.count = 0;

        for (int i = 0; i < taxa.size(); i++)
          {
            if (taxonFilter.accepts(taxa.get(i), match))
              {
                newIndexes[results.count++] = i;
              }
          }

        return results;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected void publishResults (final @Nonnull CharSequence match,
                                   final @Nonnull FilterResults filterResults)
      {
        indexTable = (int[])filterResults.values;
        filteredCount = filterResults.count;
        taxonomyAdapter.notifyDataSetChanged();
      }
  }
